Makes Dept able to track dependencies by
wait_for_completion()/complete().

In order to obtain the meaningful caller points, replace all the
wait_for_completion*() declarations with macros in the header.

Signed-off-by: Byungchul Park <byungchul.p...@lge.com>
---
 include/linux/completion.h | 89 +++++++++++++++++++++++++++++++++++++++++-----
 kernel/sched/completion.c  | 60 +++++++++++++++----------------
 2 files changed, 110 insertions(+), 39 deletions(-)

diff --git a/include/linux/completion.h b/include/linux/completion.h
index 62b32b1..0408f6d 100644
--- a/include/linux/completion.h
+++ b/include/linux/completion.h
@@ -10,6 +10,7 @@
  */
 
 #include <linux/swait.h>
+#include <linux/dept_sdt.h>
 
 /*
  * struct completion - structure used to maintain state for a "completion"
@@ -99,19 +100,89 @@ static inline void reinit_completion(struct completion *x)
        x->done = 0;
 }
 
-extern void wait_for_completion(struct completion *);
-extern void wait_for_completion_io(struct completion *);
-extern int wait_for_completion_interruptible(struct completion *x);
-extern int wait_for_completion_killable(struct completion *x);
-extern int wait_for_completion_state(struct completion *x, unsigned int state);
-extern unsigned long wait_for_completion_timeout(struct completion *x,
+extern void raw_wait_for_completion(struct completion *);
+extern void raw_wait_for_completion_io(struct completion *);
+extern int raw_wait_for_completion_interruptible(struct completion *x);
+extern int raw_wait_for_completion_killable(struct completion *x);
+extern int raw_wait_for_completion_state(struct completion *x, unsigned int 
state);
+extern unsigned long raw_wait_for_completion_timeout(struct completion *x,
                                                   unsigned long timeout);
-extern unsigned long wait_for_completion_io_timeout(struct completion *x,
+extern unsigned long raw_wait_for_completion_io_timeout(struct completion *x,
                                                    unsigned long timeout);
-extern long wait_for_completion_interruptible_timeout(
+extern long raw_wait_for_completion_interruptible_timeout(
        struct completion *x, unsigned long timeout);
-extern long wait_for_completion_killable_timeout(
+extern long raw_wait_for_completion_killable_timeout(
        struct completion *x, unsigned long timeout);
+
+#define wait_for_completion(x)                                 \
+({                                                             \
+       sdt_might_sleep_strong(NULL);                           \
+       raw_wait_for_completion(x);                             \
+       sdt_might_sleep_finish();                               \
+})
+#define wait_for_completion_io(x)                              \
+({                                                             \
+       sdt_might_sleep_strong(NULL);                           \
+       raw_wait_for_completion_io(x);                          \
+       sdt_might_sleep_finish();                               \
+})
+#define wait_for_completion_interruptible(x)                   \
+({                                                             \
+       int __ret;                                              \
+       sdt_might_sleep_strong(NULL);                           \
+       __ret = raw_wait_for_completion_interruptible(x);       \
+       sdt_might_sleep_finish();                               \
+       __ret;                                                  \
+})
+#define wait_for_completion_killable(x)                                \
+({                                                             \
+       int __ret;                                              \
+       sdt_might_sleep_strong(NULL);                           \
+       __ret = raw_wait_for_completion_killable(x);            \
+       sdt_might_sleep_finish();                               \
+       __ret;                                                  \
+})
+#define wait_for_completion_state(x, s)                                \
+({                                                             \
+       int __ret;                                              \
+       sdt_might_sleep_strong(NULL);                           \
+       __ret = raw_wait_for_completion_state(x, s);            \
+       sdt_might_sleep_finish();                               \
+       __ret;                                                  \
+})
+#define wait_for_completion_timeout(x, t)                      \
+({                                                             \
+       unsigned long __ret;                                    \
+       sdt_might_sleep_strong(NULL);                           \
+       __ret = raw_wait_for_completion_timeout(x, t);          \
+       sdt_might_sleep_finish();                               \
+       __ret;                                                  \
+})
+#define wait_for_completion_io_timeout(x, t)                   \
+({                                                             \
+       unsigned long __ret;                                    \
+       sdt_might_sleep_strong(NULL);                           \
+       __ret = raw_wait_for_completion_io_timeout(x, t);       \
+       sdt_might_sleep_finish();                               \
+       __ret;                                                  \
+})
+#define wait_for_completion_interruptible_timeout(x, t)                \
+({                                                             \
+       long __ret;                                             \
+       sdt_might_sleep_strong(NULL);                           \
+       __ret = raw_wait_for_completion_interruptible_timeout(x, t);\
+       sdt_might_sleep_finish();                               \
+       __ret;                                                  \
+})
+#define wait_for_completion_killable_timeout(x, t)             \
+({                                                             \
+       long __ret;                                             \
+       sdt_might_sleep_strong(NULL);                           \
+       __ret = raw_wait_for_completion_killable_timeout(x, t); \
+       sdt_might_sleep_finish();                               \
+       __ret;                                                  \
+})
+
 extern bool try_wait_for_completion(struct completion *x);
 extern bool completion_done(struct completion *x);
 
diff --git a/kernel/sched/completion.c b/kernel/sched/completion.c
index d57a5c1..8fcf5ee 100644
--- a/kernel/sched/completion.c
+++ b/kernel/sched/completion.c
@@ -4,7 +4,7 @@
  * Generic wait-for-completion handler;
  *
  * It differs from semaphores in that their default case is the opposite,
  * wait_for_completion default blocks whereas semaphore default non-block. The
The
  * interface also makes it easy to 'complete' multiple waiting threads,
  * something which isn't entirely natural for semaphores.
  *
@@ -20,7 +20,7 @@
  * This will wake up a single thread waiting on this completion. Threads will 
be
  * awakened in the same order in which they were queued.
  *
  * See also complete_all(), wait_for_completion() and related routines.
  *
  * If this function wakes up a task, it executes a full memory barrier before
  * accessing the task state.
@@ -124,23 +124,23 @@ void complete_all(struct completion *x)
 }
 
 /**
  * wait_for_completion: - waits for completion of a task
  * @x:  holds the state of this particular completion
  *
  * This waits to be signaled for completion of a specific task. It is NOT
  * interruptible and there is no timeout.
  *
  * See also similar routines (i.e. wait_for_completion_timeout()) with timeout
timeout
  * and interrupt capability. Also see complete().
  */
-void __sched wait_for_completion(struct completion *x)
+void __sched raw_wait_for_completion(struct completion *x)
 {
        wait_for_common(x, MAX_SCHEDULE_TIMEOUT, TASK_UNINTERRUPTIBLE);
 }
-EXPORT_SYMBOL(wait_for_completion);
+EXPORT_SYMBOL(raw_wait_for_completion);
 
 /**
  * wait_for_completion_timeout: - waits for completion of a task (w/timeout)
(w/timeout)
  * @x:  holds the state of this particular completion
  * @timeout:  timeout value in jiffies
  *
@@ -152,28 +152,28 @@ void __sched wait_for_completion(struct completion *x)
  * till timeout) if completed.
  */
 unsigned long __sched
-wait_for_completion_timeout(struct completion *x, unsigned long timeout)
+raw_wait_for_completion_timeout(struct completion *x, unsigned long timeout)
 {
        return wait_for_common(x, timeout, TASK_UNINTERRUPTIBLE);
 }
-EXPORT_SYMBOL(wait_for_completion_timeout);
+EXPORT_SYMBOL(raw_wait_for_completion_timeout);
 
 /**
  * wait_for_completion_io: - waits for completion of a task
  * @x:  holds the state of this particular completion
  *
  * This waits to be signaled for completion of a specific task. It is NOT
  * interruptible and there is no timeout. The caller is accounted as waiting
  * for IO (which traditionally means blkio only).
  */
-void __sched wait_for_completion_io(struct completion *x)
+void __sched raw_wait_for_completion_io(struct completion *x)
 {
        wait_for_common_io(x, MAX_SCHEDULE_TIMEOUT, TASK_UNINTERRUPTIBLE);
 }
-EXPORT_SYMBOL(wait_for_completion_io);
+EXPORT_SYMBOL(raw_wait_for_completion_io);
 
 /**
  * wait_for_completion_io_timeout: - waits for completion of a task (w/timeout)
(w/timeout)
  * @x:  holds the state of this particular completion
  * @timeout:  timeout value in jiffies
  *
@@ -186,14 +186,14 @@ void __sched wait_for_completion_io(struct completion *x)
  * till timeout) if completed.
  */
 unsigned long __sched
-wait_for_completion_io_timeout(struct completion *x, unsigned long timeout)
+raw_wait_for_completion_io_timeout(struct completion *x, unsigned long timeout)
 {
        return wait_for_common_io(x, timeout, TASK_UNINTERRUPTIBLE);
 }
-EXPORT_SYMBOL(wait_for_completion_io_timeout);
+EXPORT_SYMBOL(raw_wait_for_completion_io_timeout);
 
 /**
  * wait_for_completion_interruptible: - waits for completion of a task (w/intr)
(w/intr)
  * @x:  holds the state of this particular completion
  *
  * This waits for completion of a specific task to be signaled. It is
@@ -201,7 +201,7 @@ void __sched wait_for_completion_io(struct completion *x)
  *
  * Return: -ERESTARTSYS if interrupted, 0 if completed.
  */
-int __sched wait_for_completion_interruptible(struct completion *x)
+int __sched raw_wait_for_completion_interruptible(struct completion *x)
 {
        long t = wait_for_common(x, MAX_SCHEDULE_TIMEOUT, TASK_INTERRUPTIBLE);
 
@@ -209,10 +209,10 @@ int __sched wait_for_completion_interruptible(struct 
completion *x)
                return t;
        return 0;
 }
-EXPORT_SYMBOL(wait_for_completion_interruptible);
+EXPORT_SYMBOL(raw_wait_for_completion_interruptible);
 
 /**
- * wait_for_completion_interruptible_timeout: - waits for completion 
(w/(to,intr))
+ * raw_wait_for_completion_interruptible_timeout: - waits for completion 
(w/(to,intr))
  * @x:  holds the state of this particular completion
  * @timeout:  timeout value in jiffies
  *
@@ -223,15 +223,15 @@ int __sched wait_for_completion_interruptible(struct 
completion *x)
  * or number of jiffies left till timeout) if completed.
  */
 long __sched
-wait_for_completion_interruptible_timeout(struct completion *x,
+raw_wait_for_completion_interruptible_timeout(struct completion *x,
                                          unsigned long timeout)
 {
        return wait_for_common(x, timeout, TASK_INTERRUPTIBLE);
 }
-EXPORT_SYMBOL(wait_for_completion_interruptible_timeout);
+EXPORT_SYMBOL(raw_wait_for_completion_interruptible_timeout);
 
 /**
  * wait_for_completion_killable: - waits for completion of a task (killable)
(killable)
  * @x:  holds the state of this particular completion
  *
  * This waits to be signaled for completion of a specific task. It can be
@@ -239,7 +239,7 @@ int __sched wait_for_completion_interruptible(struct 
completion *x)
  *
  * Return: -ERESTARTSYS if interrupted, 0 if completed.
  */
-int __sched wait_for_completion_killable(struct completion *x)
+int __sched raw_wait_for_completion_killable(struct completion *x)
 {
        long t = wait_for_common(x, MAX_SCHEDULE_TIMEOUT, TASK_KILLABLE);
 
@@ -247,9 +247,9 @@ int __sched wait_for_completion_killable(struct completion 
*x)
                return t;
        return 0;
 }
-EXPORT_SYMBOL(wait_for_completion_killable);
+EXPORT_SYMBOL(raw_wait_for_completion_killable);
 
-int __sched wait_for_completion_state(struct completion *x, unsigned int state)
+int __sched raw_wait_for_completion_state(struct completion *x, unsigned int 
state)
 {
        long t = wait_for_common(x, MAX_SCHEDULE_TIMEOUT, state);
 
@@ -257,10 +257,10 @@ int __sched wait_for_completion_state(struct completion 
*x, unsigned int state)
                return t;
        return 0;
 }
-EXPORT_SYMBOL(wait_for_completion_state);
+EXPORT_SYMBOL(raw_wait_for_completion_state);
 
 /**
- * wait_for_completion_killable_timeout: - waits for completion of a task 
(w/(to,killable))
+ * raw_wait_for_completion_killable_timeout: - waits for completion of a task 
(w/(to,killable))
  * @x:  holds the state of this particular completion
  * @timeout:  timeout value in jiffies
  *
@@ -272,12 +272,12 @@ int __sched wait_for_completion_state(struct completion 
*x, unsigned int state)
  * or number of jiffies left till timeout) if completed.
  */
 long __sched
-wait_for_completion_killable_timeout(struct completion *x,
+raw_wait_for_completion_killable_timeout(struct completion *x,
                                     unsigned long timeout)
 {
        return wait_for_common(x, timeout, TASK_KILLABLE);
 }
-EXPORT_SYMBOL(wait_for_completion_killable_timeout);
+EXPORT_SYMBOL(raw_wait_for_completion_killable_timeout);
 
 /**
  *     try_wait_for_completion - try to decrement a completion without blocking
@@ -319,7 +319,7 @@ bool try_wait_for_completion(struct completion *x)
  *     completion_done - Test to see if a completion has any waiters
  *     @x:     completion structure
  *
  *     Return: 0 if there are waiters (wait_for_completion() in progress)
  *              1 if there are no waiters.
  *
  *     Note, this will always return true if complete_all() was called on @X.
@@ -4,7 +4,7 @@
  *
  * It differs from semaphores in that their default case is the opposite,
- * wait_for_completion default blocks whereas semaphore default non-block. The
+ * raw_wait_for_completion default blocks whereas semaphore default non-block. 
The
  * interface also makes it easy to 'complete' multiple waiting threads,
@@ -20,7 +20,7 @@
  * awakened in the same order in which they were queued.
  *
- * See also complete_all(), wait_for_completion() and related routines.
+ * See also complete_all(), raw_wait_for_completion() and related routines.
  *
  * If this function wakes up a task, it executes a full memory barrier before
@@ -124,23 +124,23 @@ void complete_all(struct completion *x)
 
 /**
- * wait_for_completion: - waits for completion of a task
+ * raw_wait_for_completion: - waits for completion of a task
  * @x:  holds the state of this particular completion
  *
@@ -124,23 +124,23 @@ void complete_all(struct completion *x)
  * interruptible and there is no timeout.
  *
- * See also similar routines (i.e. wait_for_completion_timeout()) with timeout
+ * See also similar routines (i.e. raw_wait_for_completion_timeout()) with 
timeout
  * and interrupt capability. Also see complete().
@@ -124,23 +124,23 @@ void complete_all(struct completion *x)
 
 /**
- * wait_for_completion_timeout: - waits for completion of a task (w/timeout)
+ * raw_wait_for_completion_timeout: - waits for completion of a task 
(w/timeout)
  * @x:  holds the state of this particular completion
@@ -152,28 +152,28 @@ void __sched wait_for_completion(struct completion *x)
 
 /**
- * wait_for_completion_io: - waits for completion of a task
+ * raw_wait_for_completion_io: - waits for completion of a task
  * @x:  holds the state of this particular completion
  *
@@ -152,28 +152,28 @@ void __sched wait_for_completion(struct completion *x)
 
 /**
- * wait_for_completion_io_timeout: - waits for completion of a task (w/timeout)
+ * raw_wait_for_completion_io_timeout: - waits for completion of a task 
(w/timeout)
  * @x:  holds the state of this particular completion
@@ -186,14 +186,14 @@ void __sched wait_for_completion_io(struct completion *x)
 
 /**
- * wait_for_completion_interruptible: - waits for completion of a task (w/intr)
+ * raw_wait_for_completion_interruptible: - waits for completion of a task 
(w/intr)
  * @x:  holds the state of this particular completion
@@ -223,15 +223,15 @@ int __sched wait_for_completion_interruptible(struct 
 
 /**
- * wait_for_completion_killable: - waits for completion of a task (killable)
+ * raw_wait_for_completion_killable: - waits for completion of a task 
(killable)
  * @x:  holds the state of this particular completion
@@ -319,7 +319,7 @@ bool try_wait_for_completion(struct completion *x)
  *     @x:     completion structure
  *
- *     Return: 0 if there are waiters (wait_for_completion() in progress)
+ *     Return: 0 if there are waiters (raw_wait_for_completion() in progress)
  *              1 if there are no waiters.
  *
-- 
1.9.1

Reply via email to